#include "Theme.h"

#include "NEngine/SpriteLoader.h"
#include "NEngine/SoundLoader.h"
#include "NEngine/Sound.h"
#include "Engine/Font.h"

#include <iostream>
#include <cassert>

Theme::Theme(NE::SpriteLoader* pSpriteLoader, NE::SoundLoader* pSoundLoader)
    :m_pSpriteLoader(pSpriteLoader),m_pSoundLoader(pSoundLoader)
{
    assert(pSpriteLoader);
    // assert(pSoundLoader);
}

Theme :: ~Theme()
{
    for ( std::map<std::string, Font*>::const_iterator itFont = m_fontsBank.begin() ; itFont != m_fontsBank.end() ; ++itFont )
    {
        delete (itFont->second);
    }

    for ( std::map<std::string, NE::Sound*>::const_iterator itSound = m_soundsBank.begin() ; itSound != m_soundsBank.end() ; ++itSound )
    {
        delete (itSound->second);
    }
}

bool Theme :: load()
{
    m_spritesBank["background"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/background.png");
    if ( m_spritesBank["background"] == NULL )
    {
        return false;
    }

    m_spritesBank["ball"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/ball.png");
    if ( m_spritesBank["ball"] == NULL )
    {
        return false;
    }

    m_spritesBank["safe"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/safe.png");
    if ( m_spritesBank["safe"] == NULL )
    {
        return false;
    }

    m_spritesBank["superb"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/superb.png");
    if ( m_spritesBank["superb"] == NULL )
    {
        return false;
    }

    m_spritesBank["ossom"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/ossom.png");
    if ( m_spritesBank["ossom"] == NULL )
    {
        return false;
    }

    m_spritesBank["gameover"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/gameover.png");
    if ( m_spritesBank["gameover"] == NULL )
    {
        return false;
    }

    m_spritesBank["ready"] = m_pSpriteLoader->loadSpriteFromFile("./data/gfx/ready.png");
    if ( m_spritesBank["ready"] == NULL )
    {
        return false;
    }

    m_fontsBank["font"] = new Font(m_pSpriteLoader->loadSpriteFromFile("./data/gfx/font.png"),USize2(30,30),' ');
    if ( m_fontsBank["font"] == NULL )
    {
        return false;
    }

    return true;
}

NE::Sprite* Theme :: getSprite(const std::string& name)
{
    if ( m_spritesBank.count(name) != 0 )
    {
        return m_spritesBank[name];
    }

    std::cout << "Sprite '" << name << "' not found in Theme" << std::endl;
    return NULL;
}

Font* Theme :: getFont(const std::string& name)
{
    if ( m_fontsBank.count(name) != 0 )
    {
        return m_fontsBank[name];
    }

    std::cout << "Font '" << name << "' not found in Theme" << std::endl;
    return NULL;
}

NE::Sound* Theme :: getSound(const std::string& name)
{
    if ( m_soundsBank.count(name) != 0 )
    {
        return m_soundsBank[name];
    }

    std::cout << "Sound '" << name << "' not found in Theme" << std::endl;
    return NULL;
}
