/**
OBG_Bounce is a fast release game
Copyright (C) 2011  Alexandre LAURENT

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

website: http://www.developpez.net/forums/d1149377/applications/developpement-2d-3d-jeux/projets/we-jeu-2-participation-littlewhite/#post6329974
e-mail: lw.demoscene@gmail.com
**/

#include "Ball.h"

#include "NEngine/Renderer.h"
#include "Engine/Theme.h"

#include <cstdlib>
#include <cassert>

Ball :: Ball()
{
    reset();
}

bool Ball :: load(Theme* pTheme)
{
    assert(pTheme);

    m_pSprite = pTheme->getSprite("ball");

	return true;
}

bool Ball :: draw(NE::Renderer* pRenderer)
{
    return pRenderer->drawSurface(m_position, *m_pSprite);
}

bool Ball :: update(unsigned time)
{
    static unsigned lastTime = time;
    m_position = m_position + m_direction * m_speed;
    m_speed += (time - lastTime) / 10000.0;

    lastTime = time;

    return true;
}

void Ball :: reset()
{
    m_position = IVec2(301,300);
    m_speed = 5;
    if ( rand() / static_cast<float>(RAND_MAX) < 0.5 )
    {
        if ( rand() / static_cast<float>(RAND_MAX) < 0.5 )
        {
            m_direction = IVec2(1,1);
        }
        else
        {
            m_direction = IVec2(1,-1);
        }
    }
    else
    {
        if ( rand() / static_cast<float>(RAND_MAX) < 0.5 )
        {
            m_direction = IVec2(-1,1);
        }
        else
        {
            m_direction = IVec2(-1,-1);
        }
    }
}

void Ball :: randomBounce(int* pValue)
{
    assert(pValue);


    if ( *pValue <= 0 )
    {
        if ( rand() / static_cast<float>(RAND_MAX) < 0.5 )
        {
            *pValue = 1;
        }
        else
        {
            *pValue = 2;
        }
    }
    else
    {
        if ( rand() / static_cast<float>(RAND_MAX) < 0.5 )
        {
            *pValue = -1;
        }
        else
        {
            *pValue = -2;
        }
    }
}

void Ball :: bounce()
{
    if ( m_position.x <= 600 - m_position.x )
    {
        if ( m_position.x <= m_position.y && m_position.x <= 600 - m_position.y )
        {
            randomBounce(&m_direction.x);
            return;
        }
    }
    else
    {
        if ( (600 - m_position.x) <=  (600 - m_position.y) &&  (600 - m_position.x) <= m_position.y )
        {
            randomBounce(&m_direction.x);
            return;
        }
    }

    if ( m_position.y <= 600 - m_position.y )
    {
        if ( m_position.y <= m_position.x && m_position.y <= 600 - m_position.x )
        {
            randomBounce(&m_direction.y);
            return;
        }
    }
    else
    {
        if ( (600 - m_position.y) <= (600 - m_position.x) && (600 - m_position.y) <= m_position.x )
        {
            randomBounce(&m_direction.y);
            return;
        }
    }
}
