/**
OBG_Bounce is a fast release game
Copyright (C) 2011  Alexandre LAURENT

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

website: http://www.developpez.net/forums/d1149377/applications/developpement-2d-3d-jeux/projets/we-jeu-2-participation-littlewhite/#post6329974
e-mail: lw.demoscene@gmail.com
**/

#include "Game.h"

#include "NEngine/NEngine.h"
#include "NEngine/Window.h"
#include "NEngine/Renderer.h"
#include "NEngine/Time.h"
#include "NEngine/InputManager.h"
#include "NEngine/Exceptions/FileNotFoundException.h"

#include "Engine/Font.h"

#include "Types/Colour.h"

#include <sstream>
#include <cassert>

Game :: Game(NE::NEngine* pNEngine)
    :m_pNEngine(pNEngine),m_theme(pNEngine->getSpriteLoader(),NULL),
    fpsNumber(0),fpsCounter(0),fpsLastUpdateTime(0),lastUpdateTime(0),playTime(0),m_lastButtonState(false),m_isPlaying(false)
{
    assert(pNEngine);
}

bool Game :: draw()
{
    m_pNEngine->getRenderer()->clearScreen(Colour(64,0,0));

    m_board.draw(m_pNEngine->getRenderer());

    if ( m_isPlaying )
    {
        m_ball.draw(m_pNEngine->getRenderer());
    }

    m_message.draw(m_pNEngine->getRenderer());

    {
        std::ostringstream oss;
        char prev = oss.fill ('0');
        oss.width(9);

        oss << m_score.getScore();
        oss.fill(prev);
        m_theme.getFont("font")->drawVertically(*m_pNEngine->getRenderer(),oss.str(),IVec2(676,180));
    }

    {
        std::ostringstream ossMin;
        std::ostringstream ossSec;
        char prevMin = ossMin.fill ('0');
        char prevSec = ossSec.fill ('0');
        ossMin.width(2);

        ossMin << (playTime / 60);
        ossMin.fill(prevMin);
        ossSec.width(2);
        ossSec << (playTime % 60);
        ossSec.fill(prevSec);
        m_theme.getFont("font")->drawVertically(*m_pNEngine->getRenderer(),ossMin.str() + ":" + ossSec.str(),IVec2(718,220));
    }


    if ( m_pNEngine->getRenderer()->updateWindow() == false )
    {
        std::cerr << "Fail to draw on the screen" << std::endl;
    }

    return true;
}

bool Game :: update()
{
    BoardZone bZone = BZ_Ready;

    m_pNEngine->getInputManager()->update();
    unsigned ellapsedTime = m_pNEngine->getTime()->getTime();

    NE::InputManager::Buttons buttons = m_pNEngine->getInputManager()->getButtonsPressed();

    if ( m_isPlaying )
    {
        m_ball.update(ellapsedTime);
        bZone = m_board.getZone(&m_ball);
        m_message.update(ellapsedTime);
        m_score.update();
        m_score.addBonus(bZone);
    }

    if ( bZone == BZ_Death )
    {
        m_isPlaying = false;
        m_message.setMessage(bZone,ellapsedTime);
    }

    if ( (buttons & NE::InputManager::INPUT_BUTTON) == NE::InputManager::INPUT_BUTTON )
    {
        if ( !m_lastButtonState )
        {
            if ( m_isPlaying )
            {
                m_ball.bounce();
                m_message.setMessage(bZone,ellapsedTime);
				lastPressed = ellapsedTime;
            }
            else if ( ellapsedTime - lastPressed > 1500 )
            {
                // Restart a game
                m_message.setMessage(BZ_Lame,ellapsedTime);
                m_isPlaying = true;
                m_ball.reset();
                m_score.reset();
                playTime = 0;
				lastPressed = ellapsedTime;
            }
            
            m_lastButtonState = true;
        }
    }
    else
    {
        m_lastButtonState = false;
    }

    return true;
}

bool Game :: load()
{
    bool bResult = true;

    try
    {
        bResult &= m_theme.load();
    }
    catch (FileNotFoundException& fnfe)
    {
        bResult = false;
    }

    if ( bResult == true )
    {
        m_board.load(&m_theme);
        m_ball.load(&m_theme);
        m_message.load(&m_theme);
        m_message.setMessage(BZ_Ready,0);
    }

    return bResult;
}

void Game :: run()
{
    bool bResult = true;

    while ( m_pNEngine->getInputManager()->needEscape() == false && m_pNEngine->getWindow()->needWindowClosure() == false && bResult == true )
    {
        bResult &= this->draw();
        // This solve a problem in Windows, because the events have to be updated in the same thread than the video.
        unsigned int actualTime = m_pNEngine->getTime()->getTime();
        if ( actualTime - lastUpdateTime > 25 )
        {
            bResult &= this->update();
            lastUpdateTime = m_pNEngine->getTime()->getTime();
        }

        // FPS management
        fpsCounter++;
        if ( actualTime - fpsLastUpdateTime > 1000 ) // After one second
        {
            fpsNumber= fpsCounter;
            fpsCounter = 0;
            fpsLastUpdateTime = m_pNEngine->getTime()->getTime();

            std::cout << "FPS: " << fpsNumber << std::endl;

            if ( m_isPlaying )
            {
                playTime++;
            }
        }
    }
}
